@@ -10,14 +10,14 @@ import (
 	"strings"
 	"time"
 
-	"github.com/containerd/containerd/log"
 	"github.com/containerd/containerd/platforms"
+	"github.com/containerd/log"
+	"github.com/distribution/reference"
 	"github.com/docker/distribution"
 	"github.com/docker/distribution/manifest/manifestlist"
 	"github.com/docker/distribution/manifest/ocischema"
 	"github.com/docker/distribution/manifest/schema1"
 	"github.com/docker/distribution/manifest/schema2"
-	"github.com/docker/distribution/reference"
 	"github.com/docker/distribution/registry/client/transport"
 	"github.com/docker/docker/distribution/metadata"
 	"github.com/docker/docker/distribution/xfer"
@@ -27,13 +27,11 @@ import (
 	"github.com/docker/docker/pkg/ioutils"
 	"github.com/docker/docker/pkg/progress"
 	"github.com/docker/docker/pkg/stringid"
-	"github.com/docker/docker/pkg/system"
 	refstore "github.com/docker/docker/reference"
 	"github.com/docker/docker/registry"
 	"github.com/opencontainers/go-digest"
 	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/pkg/errors"
-	"github.com/sirupsen/logrus"
 	archvariant "github.com/tonistiigi/go-archvariant"
 )
 
@@ -79,7 +77,7 @@ func (p *puller) pull(ctx context.Context, ref reference.Named) (err error) {
 	// TODO(tiborvass): was ReceiveTimeout
 	p.repo, err = newRepository(ctx, p.repoInfo, p.endpoint, p.config.MetaHeaders, p.config.AuthConfig, "pull")
 	if err != nil {
-		logrus.Warnf("Error getting v2 registry: %v", err)
+		log.G(ctx).Warnf("Error getting v2 registry: %v", err)
 		return err
 	}
 
@@ -88,18 +86,7 @@ func (p *puller) pull(ctx context.Context, ref reference.Named) (err error) {
 		return err
 	}
 
-	if err = p.pullRepository(ctx, ref); err != nil {
-		if _, ok := err.(fallbackError); ok {
-			return err
-		}
-		if continueOnError(err, p.endpoint.Mirror) {
-			return fallbackError{
-				err:         err,
-				transportOK: true,
-			}
-		}
-	}
-	return err
+	return p.pullRepository(ctx, ref)
 }
 
 func (p *puller) pullRepository(ctx context.Context, ref reference.Named) (err error) {
@@ -180,7 +167,7 @@ func (ld *layerDescriptor) DiffID() (layer.DiffID, error) {
 }
 
 func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress.Output) (io.ReadCloser, int64, error) {
-	logrus.Debugf("pulling blob %q", ld.digest)
+	log.G(ctx).Debugf("pulling blob %q", ld.digest)
 
 	var (
 		err    error
@@ -195,19 +182,19 @@ func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress
 	} else {
 		offset, err = ld.tmpFile.Seek(0, io.SeekEnd)
 		if err != nil {
-			logrus.Debugf("error seeking to end of download file: %v", err)
+			log.G(ctx).Debugf("error seeking to end of download file: %v", err)
 			offset = 0
 
 			ld.tmpFile.Close()
 			if err := os.Remove(ld.tmpFile.Name()); err != nil {
-				logrus.Errorf("Failed to remove temp file: %s", ld.tmpFile.Name())
+				log.G(ctx).Errorf("Failed to remove temp file: %s", ld.tmpFile.Name())
 			}
 			ld.tmpFile, err = createDownloadFile()
 			if err != nil {
 				return nil, 0, xfer.DoNotRetry{Err: err}
 			}
 		} else if offset != 0 {
-			logrus.Debugf("attempting to resume download of %q from %d bytes", ld.digest, offset)
+			log.G(ctx).Debugf("attempting to resume download of %q from %d bytes", ld.digest, offset)
 		}
 	}
 
@@ -215,7 +202,7 @@ func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress
 
 	layerDownload, err := ld.open(ctx)
 	if err != nil {
-		logrus.Errorf("Error initiating layer download: %v", err)
+		log.G(ctx).Errorf("Error initiating layer download: %v", err)
 		return nil, 0, retryOnError(err)
 	}
 
@@ -236,7 +223,7 @@ func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress
 		size = 0
 	} else {
 		if size != 0 && offset > size {
-			logrus.Debug("Partial download is larger than full blob. Starting over")
+			log.G(ctx).Debug("Partial download is larger than full blob. Starting over")
 			offset = 0
 			if err := ld.truncateDownloadFile(); err != nil {
 				return nil, 0, xfer.DoNotRetry{Err: err}
@@ -274,7 +261,7 @@ func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress
 
 	if !ld.verifier.Verified() {
 		err = fmt.Errorf("filesystem layer verification failed for digest %s", ld.digest)
-		logrus.Error(err)
+		log.G(ctx).Error(err)
 
 		// Allow a retry if this digest verification error happened
 		// after a resumed download.
@@ -290,13 +277,13 @@ func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress
 
 	progress.Update(progressOutput, ld.ID(), "Download complete")
 
-	logrus.Debugf("Downloaded %s to tempfile %s", ld.ID(), tmpFile.Name())
+	log.G(ctx).Debugf("Downloaded %s to tempfile %s", ld.ID(), tmpFile.Name())
 
 	_, err = tmpFile.Seek(0, io.SeekStart)
 	if err != nil {
 		tmpFile.Close()
 		if err := os.Remove(tmpFile.Name()); err != nil {
-			logrus.Errorf("Failed to remove temp file: %s", tmpFile.Name())
+			log.G(ctx).Errorf("Failed to remove temp file: %s", tmpFile.Name())
 		}
 		ld.tmpFile = nil
 		ld.verifier = nil
@@ -311,7 +298,7 @@ func (ld *layerDescriptor) Download(ctx context.Context, progressOutput progress
 		tmpFile.Close()
 		err := os.RemoveAll(tmpFile.Name())
 		if err != nil {
-			logrus.Errorf("Failed to remove temp file: %s", tmpFile.Name())
+			log.G(ctx).Errorf("Failed to remove temp file: %s", tmpFile.Name())
 		}
 		return err
 	}), size, nil
@@ -321,7 +308,7 @@ func (ld *layerDescriptor) Close() {
 	if ld.tmpFile != nil {
 		ld.tmpFile.Close()
 		if err := os.RemoveAll(ld.tmpFile.Name()); err != nil {
-			logrus.Errorf("Failed to remove temp file: %s", ld.tmpFile.Name())
+			log.G(context.TODO()).Errorf("Failed to remove temp file: %s", ld.tmpFile.Name())
 		}
 	}
 }
@@ -331,12 +318,12 @@ func (ld *layerDescriptor) truncateDownloadFile() error {
 	ld.verifier = nil
 
 	if _, err := ld.tmpFile.Seek(0, io.SeekStart); err != nil {
-		logrus.Errorf("error seeking to beginning of download file: %v", err)
+		log.G(context.TODO()).Errorf("error seeking to beginning of download file: %v", err)
 		return err
 	}
 
 	if err := ld.tmpFile.Truncate(0); err != nil {
-		logrus.Errorf("error truncating download file: %v", err)
+		log.G(context.TODO()).Errorf("error truncating download file: %v", err)
 		return err
 	}
 
@@ -375,11 +362,10 @@ func (p *puller) pullTag(ctx context.Context, ref reference.Named, platform *oci
 		return false, fmt.Errorf("internal error: reference has neither a tag nor a digest: %s", reference.FamiliarString(ref))
 	}
 
-	ctx = log.WithLogger(ctx, logrus.WithFields(
-		logrus.Fields{
-			"digest": dgst,
-			"remote": ref,
-		}))
+	ctx = log.WithLogger(ctx, log.G(ctx).WithFields(log.Fields{
+		"digest": dgst,
+		"remote": ref,
+	}))
 
 	desc := ocispec.Descriptor{
 		MediaType: mt,
@@ -390,7 +376,7 @@ func (p *puller) pullTag(ctx context.Context, ref reference.Named, platform *oci
 	manifest, err := p.manifestStore.Get(ctx, desc, ref)
 	if err != nil {
 		if isTagged && isNotFound(errors.Cause(err)) {
-			logrus.WithField("ref", ref).WithError(err).Debug("Falling back to pull manifest by tag")
+			log.G(ctx).WithField("ref", ref).WithError(err).Debug("Falling back to pull manifest by tag")
 
 			msg := `%s Failed to pull manifest by the resolved digest. This registry does not
 	appear to conform to the distribution registry specification; falling back to
@@ -428,7 +414,7 @@ func (p *puller) pullTag(ctx context.Context, ref reference.Named, platform *oci
 		}
 	}
 
-	logrus.Debugf("Pulling ref from V2 registry: %s", reference.FamiliarString(ref))
+	log.G(ctx).Debugf("Pulling ref from V2 registry: %s", reference.FamiliarString(ref))
 	progress.Message(p.config.ProgressOutput, tagOrDigest, "Pulling from "+reference.FamiliarName(p.repo.Named()))
 
 	var (
@@ -438,9 +424,12 @@ func (p *puller) pullTag(ctx context.Context, ref reference.Named, platform *oci
 
 	switch v := manifest.(type) {
 	case *schema1.SignedManifest:
-		msg := fmt.Sprintf("[DEPRECATION NOTICE] Docker Image Format v1, and Docker Image manifest version 2, schema 1 support will be removed in an upcoming release. Suggest the author of %s to upgrade the image to the OCI Format, or Docker Image manifest v2, schema 2. More information at https://docs.docker.com/go/deprecated-image-specs/", ref)
-		logrus.Warn(msg)
-		progress.Message(p.config.ProgressOutput, "", msg)
+		err := DeprecatedSchema1ImageError(ref)
+		log.G(ctx).Warn(err.Error())
+		if os.Getenv("DOCKER_ENABLE_DEPRECATED_PULL_SCHEMA_1_IMAGE") == "" {
+			return false, err
+		}
+		progress.Message(p.config.ProgressOutput, "", err.Error())
 
 		id, manifestDigest, err = p.pullSchema1(ctx, ref, v, platform)
 		if err != nil {
@@ -519,7 +508,7 @@ func (p *puller) pullSchema1(ctx context.Context, ref reference.Reference, unver
 	if platform != nil {
 		// Early bath if the requested OS doesn't match that of the configuration.
 		// This avoids doing the download, only to potentially fail later.
-		if !system.IsOSSupported(platform.OS) {
+		if err := image.CheckOS(platform.OS); err != nil {
 			return "", "", fmt.Errorf("cannot download image with operating system %q when requesting %q", runtime.GOOS, platform.OS)
 		}
 	}
@@ -703,7 +692,7 @@ func (p *puller) pullSchema2Layers(ctx context.Context, target distribution.Desc
 		if platform == nil {
 			// Early bath if the requested OS doesn't match that of the configuration.
 			// This avoids doing the download, only to potentially fail later.
-			if !system.IsOSSupported(configPlatform.OS) {
+			if err := image.CheckOS(configPlatform.OS); err != nil {
 				return "", fmt.Errorf("cannot download image with operating system %q when requesting %q", configPlatform.OS, layerStoreOS)
 			}
 			layerStoreOS = configPlatform.OS
@@ -718,8 +707,10 @@ func (p *puller) pullSchema2Layers(ctx context.Context, target distribution.Desc
 
 	// Assume that the operating system is the host OS if blank, and validate it
 	// to ensure we don't cause a panic by an invalid index into the layerstores.
-	if layerStoreOS != "" && !system.IsOSSupported(layerStoreOS) {
-		return "", system.ErrNotSupportedOperatingSystem
+	if layerStoreOS != "" {
+		if err := image.CheckOS(layerStoreOS); err != nil {
+			return "", err
+		}
 	}
 
 	if p.config.DownloadManager != nil {
@@ -843,7 +834,7 @@ func (p *puller) pullManifestList(ctx context.Context, ref reference.Named, mfst
 	if pp != nil {
 		platform = *pp
 	}
-	logrus.Debugf("%s resolved to a manifestList object with %d entries; looking for a %s match", ref, len(mfstList.Manifests), platforms.Format(platform))
+	log.G(ctx).Debugf("%s resolved to a manifestList object with %d entries; looking for a %s match", ref, len(mfstList.Manifests), platforms.Format(platform))
 
 	manifestMatches := filterManifests(mfstList.Manifests, platform)
 
@@ -869,9 +860,12 @@ func (p *puller) pullManifestList(ctx context.Context, ref reference.Named, mfst
 
 		switch v := manifest.(type) {
 		case *schema1.SignedManifest:
-			msg := fmt.Sprintf("[DEPRECATION NOTICE] Docker Image Format v1, and Docker Image manifest version 2, schema 1 support will be removed in an upcoming release. Suggest the author of %s to upgrade the image to the OCI Format, or Docker Image manifest v2, schema 2. More information at https://docs.docker.com/go/deprecated-image-specs/", ref)
-			logrus.Warn(msg)
-			progress.Message(p.config.ProgressOutput, "", msg)
+			err := DeprecatedSchema1ImageError(ref)
+			log.G(ctx).Warn(err.Error())
+			if os.Getenv("DOCKER_ENABLE_DEPRECATED_PULL_SCHEMA_1_IMAGE") == "" {
+				return "", "", err
+			}
+			progress.Message(p.config.ProgressOutput, "", err.Error())
 
 			platform := toOCIPlatform(match.Platform)
 			id, _, err = p.pullSchema1(ctx, manifestRef, v, platform)
@@ -930,7 +924,7 @@ func (p *puller) pullSchema2Config(ctx context.Context, dgst digest.Digest) (con
 	}
 	if !verifier.Verified() {
 		err := fmt.Errorf("image config verification failed for digest %s", dgst)
-		logrus.Error(err)
+		log.G(ctx).Error(err)
 		return nil, err
 	}
 
@@ -963,7 +957,7 @@ func retry(ctx context.Context, maxAttempts int, sleep time.Duration, f func(ctx
 				timer.Stop()
 				return ctx.Err()
 			case <-timer.C:
-				logrus.WithError(err).WithField("attempts", attempt+1).Debug("retrying after error")
+				log.G(ctx).WithError(err).WithField("attempts", attempt+1).Debug("retrying after error")
 				sleep *= 2
 			}
 		}
@@ -987,7 +981,7 @@ func schema2ManifestDigest(ref reference.Named, mfst distribution.Manifest) (dig
 		}
 		if !verifier.Verified() {
 			err := fmt.Errorf("manifest verification failed for digest %s", digested.Digest())
-			logrus.Error(err)
+			log.G(context.TODO()).Error(err)
 			return "", err
 		}
 		return digested.Digest(), nil
@@ -1007,7 +1001,7 @@ func verifySchema1Manifest(signedManifest *schema1.SignedManifest, ref reference
 		}
 		if !verifier.Verified() {
 			err := fmt.Errorf("image verification failed for digest %s", digested.Digest())
-			logrus.Error(err)
+			log.G(context.TODO()).Error(err)
 			return nil, err
 		}
 	}
